function fit = NLReg03(fun,bstart,x,y,param) ;
%
%
%  Gauss-Newton nonlinear least squares algorithm                           
%  fun is the regression function and must be defined in an m-file somewhere               
%  bstart = regression parameters starting values (m dimensional)
%  y = dependent variables  (n dimensional) 
%  x = independent variable vectors (n dimensional)  
% 
%  The following should be put in the structure "param".
%  tol:  iteration stops if relative improvement falls below tol 
%        relative improvement is                                 
%        (ssold - ssnew)./max([ssnew tol])   
%  maxiter = maximum number of iterations  
%  iprint : set equal to 1 to print results of each iteration                  
%  CALLS these m-files: grad2
%
%  USAGE:   fit = nlin2(fun,bstart,x,y,string,huberconst) ;
%
%  Last edited: Feb 11, 2004
%
if nargin < 5 ;
	param = '' ;
end ;



if isfield(param,'tol') == 0 ;
	tol=1e-8 ;
else ;
	tol = param.tol ;
end ;

if isfield(param,'maxiter') == 0 ;
	maxiter=200 ;
else ;
	 maxiter= param.maxiter ;
end ;

if isfield(param,'deltagrad') == 0 ;
	deltagrad=1e-9 ;
else ;
	 deltagrad= param.deltagrad ;
end ;

if isfield(param,'huberconst') == 0 ;
	huberconst=10*10 ;
else ;
	 huberconst= param.huberconst ;
end ;


if isfield(param,'iprint') == 0 ;
	iprint=0 ;
else ;
	iprint= param.iprint ;
end ;

% tol = 1e-8 ;
% maxiter = 25 ;
% iprint = 0 ;
% deltagrad=1e-9 ;   % delta for numerical gradients
%global iter sigma2 yhat residuals lambda;



if size(bstart,1) == 1 ;
    bstart=bstart';
end;
string = '' ;
iter = 1 ;
crit = 2 * tol ;   % start crit > tol so we don't stop at the first step
beta=bstart ;

while (crit > tol) & (iter <= maxiter) ;
    betaold = beta ;
    g =  grad2(fun,beta,x,deltagrad,string) ;  % compute gradient
    
    res = ( y - eval(fun) );
    hub = huber(res,huberconst) ;
  
    
    ssold = sum( hub.rho ) ;
    delta = inv(g'*g)*g'*hub.psi ;  % delta is the Gauss-Newton updating
    % direction
    isqu = 1 ;              %   Begin squeezing   
    ok = 0 ;   % ok = 0 means we accepted the update
    while (isqu <= 10) & (ok==0) ;
        beta = betaold + delta ;
        res = y - eval(fun) ;
        hub = huber(res,huberconst) ;
        ssnew = sum( hub.rho) ;
        if ssnew < ssold ;  %  if the criterion is met then update
            beta = betaold + delta ;     %  update
            ok = 1 ;            %  ok = 1 since we updated
        else ;
            delta = delta./2 ;  % half delta (called "squeezing")
            % if we don't update; then try
            % again
            isqu = isqu + 1 ;   % count the number of "squeeze"
        end;
    end ;                  %   End squeezing   
    
    if isqu == 11 ; crit = 0 ;  % Updata crit
    else ;
        crit = (ssold - ssnew) / max([tol; ssnew]) ; % we will stop if
        % crit is less than tol
    end ;
    ssold = ssnew ;             % update the sum of squares
    if iprint == 1 ;
        
        '         iteration =  '; % Print summary of last iteration
        iter 
        '                 beta =  ';
        beta'
        'number of squeezes =  ';
        isqu 
        '    sum of squares = ';
        ssnew
        'stopping criterion =  ' ;
        crit
    end ;
    iter = iter + 1 ;
end ;
n = size(y,1) ;
p = length(beta) ;
sigma2 = sum(res.^2)./(n-size(beta,1)) ;
sigma = sqrt(sigma2) ;
varmatrix = sigma2.* inv(g'*g) ;
hi =((g*inv(g'*g)).*g)*ones(p,1) ;
stderror = sqrt(diag(varmatrix)) ;
t = beta./stderror ;
yhat = eval(fun) ;
residuals = y - yhat ;
stdresid = residuals./(sigma.*(1-hi)) ;
mse = residuals'*residuals ./ (n - p) ;


fit = struct('beta',beta,'sigma2',sigma2,'stderror',stderror,'yhat',yhat,...
    'residuals',residuals,'mse',mse,'t',t,'varmatrix',varmatrix,'g',g,'hi',hi,'stdresid',stdresid) ;


